// Filename		: XMLSerializer.h
// Author		: Siddharth Barman
// Date			: Aug 06 2005
// Description	: Declares CXMLSerializer class implementing the ISerializer
//				  interface allowing it to act as a serialization provider. It
//				  provides serialization capability by making use of XML files
//				  for persistence.
//------------------------------------------------------------------------------

#pragma once
#include "Serializer.h"
#import <msxml4.dll> rename_namespace("MSXML") named_guids
#define UNSPEC_ERROR _T("Unspecified error.")
using namespace MSXML;
//------------------------------------------------------------------------------
#define DESERIALIZATION_FAIL	-1

class CXMLSerializer : public ISerializer
{
public:
	CXMLSerializer();
	CXMLSerializer(const CString& sFile, const CString& sAppName, bool bLoad);
	~CXMLSerializer();

	void	ReadFile();

	void	SetFile(const CString&);
	CString GetFile();

	void	SetApplicationName(const CString&);
	CString GetApplicationName();

	bool	Serialize(ISerializable*);		// a single item
	bool	Serialize(const CPtrList&);		// a list of items
	int		Deserialize(IObjectFactory*, CPtrList& objList); // list of items [out]

private:
	CString m_sFile;
	CString m_sAppName;
	MSXML::IXMLDOMDocument2Ptr m_doc;

	bool GetProperty(MSXML::IXMLDOMNode*, CProperty&);
								  // specify the name in the property object itself
	
	bool SetProperty(MSXML::IXMLDOMNode*, const CProperty& property, 
					 bool bCreateNewNode = false);
	
	bool SetAttributeValue(MSXML::IXMLDOMNode*, const CString& sAttName, 
						   const CString& sValue);
	
	bool GetAttributeValue(MSXML::IXMLDOMNode* node, const CString& sAttName,
						   CString& sValue);
	
	bool SerializeObject(ISerializable*, MSXML::IXMLDOMNode* parent=NULL); 

	int DeserializeObject(IObjectFactory*, CPtrList& objList, 
						  MSXML::IXMLDOMNode* parent=NULL); 
	
	MSXML::IXMLDOMNode* CreateChildNode(const CString& sNodeName,
										MSXML::IXMLDOMNode* = NULL);

	CString GetPlural(const CString&);
};
//------------------------------------------------------------------------------

class CXMLException : public CException
{
private:
	CString m_sMessage;

public:
	CXMLException(const CString& sMsg);
	CXMLException(const CXMLException&);
	CString ToString();
	BOOL GetErrorMessage(LPTSTR lpszError, UINT nMaxError, 
						 PUINT pnHelpContext = NULL);
	int ReportError(UINT nType = MB_OK, UINT nMessageID = 0);
};
